/*
    This file is part of ExeDefender.

    ExeDefender is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ExeDefender is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ExeDefender.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "Resource.h"
#include "Main.h"

/*
	Draw an image onto the window.
*/
void DrawImg(int iX, int iY, int iWidth, int iHeight, void *pImg, HDC hDC)
{
	HDC hDCMem = NULL;
	HBITMAP hBitmapOld = NULL;

	hDCMem = CreateCompatibleDC(hDC);
	hBitmapOld = (HBITMAP)SelectObject(hDCMem, (HBITMAP)pImg);
	BitBlt(hDC, iX, iY, iWidth, iHeight, hDCMem, 0, 0, SRCCOPY);
	SelectObject(hDCMem, hBitmapOld);
	DeleteDC(hDCMem);
}

/*
	Redraw screen.
*/
void Redraw(HWND hWnd)
{
	InvalidateRect(hWnd, NULL, true);
	UpdateWindow(hWnd);
}

/*
	Redraw the screen at the specified area.
*/
void RedrawArea(HWND hWnd, int iX, int iY, int iWidth, int iHeight)
{
	RECT Rect;

	Rect.left = iX;
	Rect.top = iY;
	Rect.right = iX + iWidth;
	Rect.bottom = iY + iHeight;
	InvalidateRect(hWnd, &Rect, true);
	UpdateWindow(hWnd);
}

/*
	Fade the main window in.
*/
DWORD WINAPI FadeInThread(LPVOID lpArg)
{
	ShowWindow((HWND)lpArg, SW_SHOWNORMAL);

	for(int i = 0; i < 255; i++)
	{
		pSetLayeredWindowAttributes((HWND)lpArg, RGB(0xc8, 0xc8, 0xc8), i, LWA_ALPHA | 1);
		Sleep(2);
	}

	return 0;
}

/*
	Fade the main window out.
*/
void FadeOut(HWND hDlg)
{
	for(int i = 0; i < 255; i++)
	{
		pSetLayeredWindowAttributes(hDlg, RGB(0xc8, 0xc8, 0xc8), 255 - i, LWA_ALPHA | 1);
		Sleep(2);
	}
}

HWND TextBox(int iX, int iY, int iWidth, int iHeight, char *pText, HWND hParent, bool bCenter)
{
	if(bCenter)
	{
		return CreateWindow("static", pText, WS_CHILD | WS_VISIBLE | SS_CENTER, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
	}
	else
	{
		return CreateWindow("static", pText, WS_CHILD | WS_VISIBLE, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
	}
}

HWND EditBox(int iX, int iY, int iWidth, int iHeight, char *pText, HWND hParent)
{
	return CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", pText, WS_CHILD | WS_VISIBLE | ES_READONLY | ES_AUTOHSCROLL, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
}

HWND Button(int iX, int iY, int iWidth, int iHeight, char *pText, HWND hParent)
{
	return CreateWindow("button", pText, WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
}

HWND CheckBox(int iX, int iY, int iWidth, int iHeight, char *pText, HWND hParent)
{
	return CreateWindow("button", pText, WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
}

HWND Slider(int iX, int iY, int iWidth, int iHeight, HWND hParent)
{
	return CreateWindow(TRACKBAR_CLASS, "", WS_CHILD | WS_VISIBLE | TBS_AUTOTICKS | TBS_NOTICKS, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
}

HWND ListBox(int iX, int iY, int iWidth, int iHeight, HWND hParent, bool bMultiSelect)
{
	if(bMultiSelect)
	{
		return CreateWindow("listbox", NULL, (WS_CHILD | WS_VISIBLE | WS_HSCROLL | LBS_STANDARD | LBS_MULTIPLESEL) & ~LBS_SORT, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
	}
	else
	{
		return CreateWindow("listbox", NULL, (WS_CHILD | WS_VISIBLE | WS_HSCROLL | LBS_STANDARD) & ~LBS_SORT, iX, iY, iWidth, iHeight, hParent, NULL, ghInstance, NULL);
	}
}

/*
	Check if a specific import is selected in the list.
*/
bool GetImportSel(int iIndex)
{
	int iRet = 0;

	iRet = SendMessage(hImportLogWnd, LB_GETSEL, iIndex, 0);

	if(iRet == 0 || iRet == LB_ERR)
	{
		return false;
	}

	return true;
}

/*
	Add line to the main log file.
*/
void AddLog(char *pText)
{
	int iEntryId = 0;
	HDC hDC = NULL;
	HFONT hFont = NULL;
	tagSIZE OutSize;
	DWORD dwCurrWidth = 0;

	iEntryId = SendMessage(hLogWnd, LB_ADDSTRING, 0, (DWORD)pText);
	SendMessage(hLogWnd, WM_VSCROLL, SB_BOTTOM, 0);

	hDC = GetDC(hLogWnd);
	hFont = (HFONT)SendMessage(hLogWnd, WM_GETFONT, 0, 0);
	SelectObject(hDC, hFont);

	memset((void*)&OutSize, 0, sizeof(OutSize));
	GetTextExtentPoint32(hDC, pText, strlen(pText), &OutSize);
	dwCurrWidth = OutSize.cx;

	dwCurrWidth += 10;

	if(dwCurrWidth > dwListBoxMaxWidth)
	{
		SendMessage(hLogWnd, LB_SETHORIZONTALEXTENT, dwCurrWidth, 0);
		dwListBoxMaxWidth = dwCurrWidth;
	}

	RedrawArea(ghWndTestWindow, 20, 442, 410, 112);
}

/*
	Add import to the list.
*/
void AddImportLog(char *pText)
{
	int iEntryId = 0;
	HDC hDC = NULL;
	HFONT hFont = NULL;
	tagSIZE OutSize;
	DWORD dwCurrWidth = 0;

	iEntryId = SendMessage(hImportLogWnd, LB_ADDSTRING, 0, (DWORD)pText);
	SendMessage(hImportLogWnd, WM_VSCROLL, SB_BOTTOM, 0);

	hDC = GetDC(hImportLogWnd);
	hFont = (HFONT)SendMessage(hImportLogWnd, WM_GETFONT, 0, 0);
	SelectObject(hDC, hFont);

	memset((void*)&OutSize, 0, sizeof(OutSize));
	GetTextExtentPoint32(hDC, pText, strlen(pText), &OutSize);
	dwCurrWidth = OutSize.cx;

	dwCurrWidth += 10;

	if(dwCurrWidth > dwImportListBoxMaxWidth)
	{
		SendMessage(hImportLogWnd, LB_SETHORIZONTALEXTENT, dwCurrWidth, 0);
		dwImportListBoxMaxWidth = dwCurrWidth;
	}
}

/*
	Redraw the screen at the slider text location, to update the text.
*/
void UpdateSliderText(HWND hWnd, DWORD dw)
{
	char szTemp[512];

	memset(szTemp, 0, sizeof(szTemp));
	_snprintf(szTemp, sizeof(szTemp) - 1, "(%d)", dw);
	SetWindowText(hWnd, szTemp);
}

/*
	Redraw the screen at the cross location, to update the graphic.
*/
void RedrawCross(HWND hWnd)
{
	RedrawArea(hWnd, WIN_X - 25, 10, 15, 15);
}

/*
	Remove all entries from import list.
*/
void ClearImportLog()
{
	SendMessage(hImportLogWnd, LB_RESETCONTENT, 0, 0);
	SendMessage(hImportLogWnd, LB_SETHORIZONTALEXTENT, 0, 0);

	dwImportListBoxMaxWidth = 0;
}

/*
	Clear all fields.
*/
void ClearAll()
{
	ClearImportLog();
	SetWindowText(hPathWnd, "");
	EnableWindow(hProtectWnd, false);
}

/*
	Map input exe into memory, and read basic details.
*/
int LoadInputFile(char *pFilename)
{
	DWORD dwImageSize = 0;
	DWORD dwFileSize = 0;
	IMAGE_NT_HEADERS *pNtHeader = NULL;
	char szTemp[512];

	ClearImportLog();

	if(pImageData)
	{
		VirtualFree(pImageData, 0, MEM_RELEASE);
		pImageData = NULL;
	}

	pImageData = GetExeData(pFilename, (DWORD*)&dwImageSize, (DWORD*)&pNtHeader, (DWORD*)&dwFileSize);
	if(pImageData == NULL)
	{
		AddLog("[ERROR] Failed to load executable.");
		AddLog("Failed.");
		return 0;
	}

	gdwImageSize = dwImageSize;
	gpNtHeader = pNtHeader;
	gdwFileSize = dwFileSize;

	PrintImports((IMAGE_DOS_HEADER*)pImageData, pNtHeader);

	memset(szTemp, 0, sizeof(szTemp));
	_snprintf(szTemp, sizeof(szTemp) - 1, "Loaded executable (%d bytes).", dwFileSize);
	AddLog(szTemp);

	return 1;
}

/*
	Select all entries in the import list.
*/
void SelectAllImports()
{
	DWORD dwCount = 0;

	dwCount = SendMessage(hImportLogWnd, LB_GETCOUNT, 0, 0);

	for(DWORD i = 0; i < dwCount; i++)
	{
		SendMessage(hImportLogWnd, LB_SETSEL, 1, i);
	}
}

/*
	Remove all selections in the import list.
*/
void SelectNoneImports()
{
	DWORD dwCount = 0;

	dwCount = SendMessage(hImportLogWnd, LB_GETCOUNT, 0, 0);

	for(DWORD i = 0; i < dwCount; i++)
	{
		SendMessage(hImportLogWnd, LB_SETSEL, 0, i);
	}
}

/*
	Window callback.
*/
LRESULT CALLBACK MainWindowProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc = NULL;
	HDC hColourDC = NULL;
	int iClickPosX = 0;
	int iClickPosY = 0;
	DWORD dwTemp = 0;
	OPENFILENAME OpenFilename;
	bool bRemoveRelocs = false;
	bool bRemovePe = false;
	bool bAntiDisassemble = false;
	bool bAntiDump = false;
	bool bAntiDebug = false;
	bool bRemoveSegmentNames = false;
	char szCurrFileName[512];
	char *pTemp1 = NULL;
	char *pTemp2 = NULL;
	char szTempFilePath[512];

	if(iMsg == WM_CREATE)
	{
		pSetLayeredWindowAttributes(hWnd, 0, 0, LWA_ALPHA);

		//get exe directory
		memset(szCurrFileName, 0, sizeof(szCurrFileName));
		GetModuleFileName(NULL, szCurrFileName, sizeof(szCurrFileName) - 1);
		pTemp1 = szCurrFileName;
		for(;;)
		{
			pTemp2 = strstr(pTemp1, "\\");
			if(pTemp2 == NULL)
			{
				break;
			}

			pTemp1 = pTemp2;
			pTemp1++;
		}

		*(char*)pTemp1 = '\0';

		//Open bitmap files, to draw the controls.
		memset(szTempFilePath, 0, sizeof(szTempFilePath));
		_snprintf(szTempFilePath, sizeof(szTempFilePath) - 1, "%sbg1.dat", pTemp1);
		pMainImg = (void*)LoadImage(ghInstance, szTempFilePath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
		if(pMainImg == NULL)
		{
			MessageBox(NULL, "Error: Failed to open bg1.dat.", "ExeDefender", MB_OK);
			ExitProcess(0);
		}

		memset(szTempFilePath, 0, sizeof(szTempFilePath));
		_snprintf(szTempFilePath, sizeof(szTempFilePath) - 1, "%scrs.dat", pTemp1);
		pCrossImg = (void*)LoadImage(ghInstance, szTempFilePath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
		if(pCrossImg == NULL)
		{
			MessageBox(NULL, "Error: Failed to open crs.dat.", "ExeDefender", MB_OK);
			ExitProcess(0);
		}

		memset(szTempFilePath, 0, sizeof(szTempFilePath));
		_snprintf(szTempFilePath, sizeof(szTempFilePath) - 1, "%scrsdwn.dat", pTemp1);
		pCrossDownImg = (void*)LoadImage(ghInstance, szTempFilePath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
		if(pCrossDownImg == NULL)
		{
			MessageBox(NULL, "Error: Failed to open crsdwn.dat.", "ExeDefender", MB_OK);
			ExitProcess(0);
		}

		pCurCrossImg = pCrossImg;

		TextBox(13, 40, 100, 20, "Input File:", hWnd, false);
		hPathWnd = EditBox(88, 38, 250, 20, "", hWnd);
		hBrowseWnd = Button(345, 38, 80, 21, "Browse", hWnd);

		TextBox(13, 80, 150, 20, "Optional Protection:", hWnd, false);
		hRemoveSegmentsWnd = CheckBox(20, 105, 200, 20, "Hide Segment Names", hWnd);
		hRemoveRelocsWnd = CheckBox(20, 125, 200, 20, "Remove Relocations", hWnd);
		hRemovePeWnd = CheckBox(20, 145, 200, 20, "Remove PE Header Fields", hWnd);
		hAntiDisassembleWnd = CheckBox(20, 165, 200, 20, "Anti-Disassemble", hWnd);
		hAntiDebugWnd = CheckBox(20, 185, 200, 20, "Anti-Debug", hWnd);
		hAntiDumpWnd = CheckBox(20, 205, 200, 20, "Anti-Dump", hWnd);

		TextBox(13, 250, 150, 20, "Protection Layers:", hWnd, false);
		hProtectionLayersSliderWnd = Slider(20, 275, 150, 20, hWnd);
		hProtectionLayersSliderTextWnd = TextBox(75, 295, 40, 20, "(0)", hWnd, true);

		TextBox(13, 325, 150, 20, "Junk Code Frequency:", hWnd, false);
		hJunkFrequencySliderWnd = Slider(20, 350, 150, 20, hWnd);
		hJunkFrequencySliderTextWnd = TextBox(75, 370, 40, 20, "(0)", hWnd, true);

		TextBox(260, 325, 150, 20, "Encryption Iterations:", hWnd, false);
		hEncryptionIterationsSliderWnd = Slider(267, 350, 150, 20, hWnd);
		hEncryptionIterationsSliderTextWnd = TextBox(75 + 260 - 13, 370, 40, 20, "(0)", hWnd, true);

		TextBox(260, 80, 150, 20, "Encrypt Imports:", hWnd, false);

		hImportLogWnd = ListBox(267, 105, 160, 140, hWnd, true);
		hSelectAllWnd = Button(268, 245, 50, 21, "All", hWnd);
		hSelectNoneWnd = Button(377, 245, 50, 21, "None", hWnd);

		hProtectWnd = Button(180, 410, 90, 20, "Protect!", hWnd);

		hLogWnd = ListBox(20, 442, 410, 112, hWnd, false);

		//=============================================================
		//Do slider stuff
		SendMessage(hProtectionLayersSliderWnd, TBM_SETRANGEMIN, 1, 1);
		SendMessage(hProtectionLayersSliderWnd, TBM_SETRANGEMAX, 1, 500);
		SendMessage(hProtectionLayersSliderWnd, TBM_SETPOS, 1, 10);
		dwProtectionLayers = 10;
		UpdateSliderText(hProtectionLayersSliderTextWnd, dwProtectionLayers);

		SendMessage(hEncryptionIterationsSliderWnd, TBM_SETRANGEMIN, 1, 1);
		SendMessage(hEncryptionIterationsSliderWnd, TBM_SETRANGEMAX, 1, 250);
		SendMessage(hEncryptionIterationsSliderWnd, TBM_SETPOS, 1, 20);
		dwEncryptionIterations = 20;
		UpdateSliderText(hEncryptionIterationsSliderTextWnd, dwEncryptionIterations);

		SendMessage(hJunkFrequencySliderWnd, TBM_SETRANGEMIN, 1, 1);
		SendMessage(hJunkFrequencySliderWnd, TBM_SETRANGEMAX, 1, 100);
		SendMessage(hJunkFrequencySliderWnd, TBM_SETPOS, 1, 20);
		dwJunkFrequency = 20;
		UpdateSliderText(hJunkFrequencySliderTextWnd, dwJunkFrequency);

		//=============================================================
		//Print stuff

		CreateThread(NULL, 0, FadeInThread, (LPVOID)hWnd, 0, &dwTemp);

		AddLog("ExeDefender v1.0, by InternalBytes Software.");
		AddLog("http://www.internalbytes.net/");
		AddLog("---------------------");
		AddLog("Ready - Please select an input file.");

		ClearAll();

		return 1;
	}
	else if(iMsg == WM_HSCROLL)
	{
		if((HWND)lParam == hProtectionLayersSliderWnd)
		{
			dwProtectionLayers = SendMessage(hProtectionLayersSliderWnd, TBM_GETPOS, 0, 0);

			UpdateSliderText(hProtectionLayersSliderTextWnd, dwProtectionLayers);

			RedrawArea(hWnd, 75, 295, 40, 20);
		}
		if((HWND)lParam == hEncryptionIterationsSliderWnd)
		{
			dwEncryptionIterations = SendMessage(hEncryptionIterationsSliderWnd, TBM_GETPOS, 0, 0);

			UpdateSliderText(hEncryptionIterationsSliderTextWnd, dwEncryptionIterations);

			RedrawArea(hWnd, 75 + 260 - 13, 370, 40, 20);
		}
		if((HWND)lParam == hJunkFrequencySliderWnd)
		{
			dwJunkFrequency = SendMessage(hJunkFrequencySliderWnd, TBM_GETPOS, 0, 0);

			UpdateSliderText(hJunkFrequencySliderTextWnd, dwJunkFrequency);

			RedrawArea(hWnd, 75, 370, 40, 20);
		}
	}
	else if(iMsg == WM_CTLCOLORSTATIC)
	{
		hColourDC = (HDC)wParam;
		if((HWND)lParam != hPathWnd && (HWND)lParam != hProtectionLayersSliderWnd && (HWND)lParam != hLogWnd && (HWND)lParam != hEncryptionIterationsSliderWnd && (HWND)lParam != hJunkFrequencySliderWnd)
		{
			SetBkMode(hColourDC, TRANSPARENT);
			SetTextColor(hColourDC, 0xFFFFFF);
			return (LRESULT)GetStockObject(NULL_PEN);
		}
		else if((HWND)lParam == hProtectionLayersSliderWnd || (HWND)lParam == hJunkFrequencySliderWnd || (HWND)lParam == hEncryptionIterationsSliderWnd)
		{
			SetBkMode(hColourDC, TRANSPARENT);
			SetBkColor(hColourDC, RGB(0x33, 0x33, 0x33));
			return (LRESULT)CreateSolidBrush(RGB(0x33, 0x33, 0x33));
		}
	}
	else if(iMsg == WM_LBUTTONDOWN)
	{
		if(wParam == MK_LBUTTON)
		{
			iClickPosX = LOWORD(lParam);
			iClickPosY = HIWORD(lParam);

			if(iClickPosY < 29)
			{
				//on caption bar!

				if((iClickPosX >= WIN_X - 25) && (iClickPosX <= WIN_X - 25 + 15) && (iClickPosY >= 10) && (iClickPosY <= 10 + 15))
				{
					bCrossDown = true;
					pCurCrossImg = pCrossDownImg;

					RedrawCross(hWnd);
				}
				else
				{
					bDragging = true;
				}
			}
		}

		return 1;
	}
	else if(iMsg == WM_LBUTTONUP)
	{
		iClickPosX = LOWORD(lParam);
		iClickPosY = HIWORD(lParam);

		if(iClickPosY < 29)
		{
			//on caption bar!

			if((iClickPosX >= WIN_X - 25) && (iClickPosX <= WIN_X - 25 + 15) && (iClickPosY >= 10) && (iClickPosY <= 10 + 15))
			{
				if(bCrossDown)
				{
					FadeOut(hWnd);
					ExitProcess(0);
				}
			}
		}

		if(bCrossDown)
		{
			bCrossDown = false;
			pCurCrossImg = pCrossImg;

			//redraw
			RedrawCross(hWnd);
		}

		bDragging = false;
		return 1;
	}
	else if(iMsg == WM_PAINT)
    {
		hdc = BeginPaint(hWnd, &ps);

		DrawImg(0, 0, WIN_X, WIN_Y, pMainImg, hdc);
		DrawImg(WIN_X - 25, 10, 15, 15, pCurCrossImg, hdc);

		EndPaint(hWnd, &ps);

		return 1;
    }
	else if(iMsg == WM_MOUSEMOVE)
	{
		iClickPosX = LOWORD(lParam);
		iClickPosY = HIWORD(lParam);

		if(iClickPosY < 29)
		{
			//on caption bar!

			if(bCrossDown)
			{
				if((iClickPosX >= WIN_X - 25) && (iClickPosX <= WIN_X - 25 + 15) && (iClickPosY >= 10) && (iClickPosY <= 10 + 15))
				{
					if(bCrossIconDown == false)
					{
						bCrossIconDown = true;
						pCurCrossImg = pCrossDownImg;

						RedrawCross(hWnd);
					}
				}
				else
				{
					if(bCrossIconDown)
					{
						bCrossIconDown = false;
						pCurCrossImg = pCrossImg;

						RedrawCross(hWnd);
					}
				}
			}
		}

		return 1;
	}
	else if(iMsg == WM_ERASEBKGND)
	{
		return 1;
	}
	else if(iMsg == WM_CLOSE)
	{
		FadeOut(hWnd);
		ExitProcess(0);
		return 0;
	}
	else if(iMsg == WM_COMMAND)
	{
		if((HWND)lParam == hBrowseWnd)
		{
			memset((void*)&OpenFilename, 0, sizeof(OpenFilename));
			memset(szFilename, 0, sizeof(szFilename));

			OpenFilename.lpstrFile = szFilename;
			OpenFilename.nMaxFile = sizeof(szFilename);
			OpenFilename.lpstrFilter = "Exe Files\0*.exe\0\0";

			OpenFilename.lStructSize = sizeof(OPENFILENAME);
			if(GetOpenFileName(&OpenFilename) == 0)
			{
				return 0;
			}

			SetWindowText(hPathWnd, szFilename);

			if(LoadInputFile(szFilename))
			{
				EnableWindow(hProtectWnd, true);
			}
		}
		else if((HWND)lParam == hProtectWnd)
		{
			if(pImageData)
			{
				memset(szBackupFilename, 0, sizeof(szBackupFilename));
				_snprintf(szBackupFilename, sizeof(szBackupFilename) - 1, "%s.bak", szFilename);
				if(MoveFile(szFilename, szBackupFilename) == 0)
				{
					AddLog("[ERROR] Failed to backup original file.");
					AddLog("Failed.");
					return 0;
				}

				if(SendMessage(hRemoveRelocsWnd, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					bRemoveRelocs = true;
				}
				if(SendMessage(hRemovePeWnd, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					bRemovePe = true;
				}
				if(SendMessage(hAntiDisassembleWnd, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					bAntiDisassemble = true;
				}
				if(SendMessage(hAntiDumpWnd, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					bAntiDump = true;
				}
				if(SendMessage(hAntiDebugWnd, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					bAntiDebug = true;
				}
				if(SendMessage(hRemoveSegmentsWnd, BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					bRemoveSegmentNames = true;
				}

				AddLog("Encrypting...");

				if(DoProtect(szBackupFilename, szFilename, bAntiDebug, bAntiDump, bAntiDisassemble, bRemoveRelocs, bRemovePe, bRemoveSegmentNames, gdwImageSize, gpNtHeader, gdwFileSize, dwProtectionLayers) == 0)
				{
					ClearAll();
					AddLog("Failed.");

					if(pImageData)
					{
						VirtualFree(pImageData, 0, MEM_RELEASE);
						pImageData = NULL;
					}

					return 0;
				}

				AddLog("Done.");
			}
		}
		else if((HWND)lParam == hSelectAllWnd)
		{
			SelectAllImports();
		}
		else if((HWND)lParam == hSelectNoneWnd)
		{
			SelectNoneImports();
		}
	}

	return DefWindowProc(hWnd, iMsg, wParam, lParam);
}

/*
	Allow the custom window to be dragged using the mouse.
*/
DWORD WINAPI DragThread(LPVOID)
{
	RECT Rect;
	POINT Point;
	int iCurrX = 0;
	int iCurrY = 0;
	int iLastX = 0;
	int iLastY = 0;

	for(;;)
	{
		GetCursorPos(&Point);

		if(bDragging)
		{
			iCurrX = Point.x;
			iCurrY = Point.y;

			GetWindowRect(ghWndTestWindow, &Rect);
			Rect.top += (iCurrY - iLastY);
			Rect.bottom += (iCurrY - iLastY);

			Rect.left += (iCurrX - iLastX);
			Rect.right += (iCurrX - iLastX);

			SetWindowPos(ghWndTestWindow, HWND_TOP, Rect.left, Rect.top, Rect.right - Rect.left, Rect.bottom - Rect.top, 0);
		}

		iLastX = Point.x;
		iLastY = Point.y;

		Sleep(10);
	}

	return 0;
}

/*
	WinMain.
*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	DWORD dwTemp = 0;
	MSG Msg;
	WNDCLASSEX wcx;
	char szWindowTitle[] = "ExeDefender v1.0";

	ghInstance = hInstance;

	memset((void*)&wcx, 0, sizeof(wcx));

	wcx.cbSize = sizeof(wcx);
	wcx.style = CS_HREDRAW | CS_VREDRAW;
	wcx.lpfnWndProc = MainWindowProc;
	wcx.cbClsExtra = 0;
	wcx.cbWndExtra = 0;
	wcx.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
	wcx.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
	wcx.hInstance = hInstance;
	wcx.hCursor = (HCURSOR) LoadImage(0, (char *)32512, IMAGE_CURSOR, 0, 0, LR_SHARED);
	wcx.hbrBackground = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
	wcx.lpszMenuName =  NULL;
	wcx.lpszClassName = "ExeDefender";

	RegisterClassEx(&wcx);
	InitCommonControls();

	pSetLayeredWindowAttributes = (int (__stdcall *)(struct HWND__ *,unsigned long,unsigned char,unsigned long))GetProcAddress(LoadLibrary("user32.dll"), "SetLayeredWindowAttributes");

	ghWndTestWindow = CreateWindowEx(0, "ExeDefender", szWindowTitle, (WS_POPUP | WS_EX_LAYERED) & ~WS_VISIBLE, (GetSystemMetrics(SM_CXFULLSCREEN) / 2) - (WIN_X / 2), (GetSystemMetrics(SM_CYFULLSCREEN) / 2) - (WIN_Y / 2), WIN_X, WIN_Y, NULL, NULL, hInstance, NULL);

	ShowWindow(ghWndTestWindow, SW_HIDE);
	UpdateWindow(ghWndTestWindow);

	SetActiveWindow(ghWndTestWindow);

	SetWindowLong(ghWndTestWindow, GWL_EXSTYLE, GetWindowLong(ghWndTestWindow, GWL_EXSTYLE) | WS_EX_LAYERED);
	pSetLayeredWindowAttributes(ghWndTestWindow, RGB(0xc8, 0xc8, 0xc8), 0, 1);

	CreateThread(NULL, 0, DragThread, 0, 0, &dwTemp);

	while(GetMessage(&Msg, NULL, 0,0) > 0)
	{
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
	}

	return 0;
}
